#include <RCSwitch.h>
#include <LiquidCrystal_I2C.h>       // library for I@C interface

#define I2C_ADDR  0x27               // Address for LCD
#define RS_pin    0                  // configuration of pins for the interface LCM1602
#define RW_pin    1
#define EN_pin    2
#define BACK_pin  3
#define D4_pin    4
#define D5_pin    5
#define D6_pin    6
#define D7_pin    7

LiquidCrystal_I2C lcd(I2C_ADDR, EN_pin, RW_pin, RS_pin, D4_pin, D5_pin, D6_pin, D7_pin, BACK_pin, POSITIVE);

const char custom[][8] PROGMEM = {
     {0x01, 0x07, 0x0F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F},      // char 1: bottom right triangle
     {0x00, 0x00, 0x00, 0x00, 0x1F, 0x1F, 0x1F, 0x1F},      // char 2: bottom block
     {0x10, 0x1C, 0x1E, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F},      // char 3: bottom left triangle
     {0x1F, 0x0F, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00},      // char 4: top right triangle
     {0x1F, 0x1E, 0x1C, 0x10, 0x00, 0x00, 0x00, 0x00},      // char 5: top left triangle
     {0x1F, 0x1F, 0x1F, 0x1F, 0x00, 0x00, 0x00, 0x00},      // char 6: upper block
     {0x1F, 0x1F, 0x1E, 0x1C, 0x18, 0x18, 0x10, 0x10},      // char 7: full top right triangle
     {0x01, 0x07, 0x0F, 0x1F, 0x00, 0x00, 0x00, 0x00}       // char 8: top right triangle
};
 

const char bn4[][30] PROGMEM = {                            // 4-line numbers
//         0               1               2               3               4              5               6                7               8               9
    {0x01,0x06,0x03, 0x08,0xFF,0xFE, 0x08,0x06,0x03, 0x08,0x06,0x03, 0xFF,0xFE,0xFF, 0xFF,0x06,0x06, 0x01,0x06,0x03, 0x06,0x06,0xFF, 0x01,0x06,0x03, 0x01,0x06,0x03},
    {0xFF,0xFE,0xFF, 0xFE,0xFF,0xFE, 0x02,0x02,0xFF, 0xFE,0x02,0xFF, 0xFF,0x02,0xFF, 0xFF,0x02,0x02, 0xFF,0x02,0x02, 0xFE,0x01,0x07, 0xFF,0x02,0xFF, 0xFF,0x02,0xFF},
    {0xFF,0xFE,0xFF, 0xFE,0xFF,0xFE, 0xFF,0xFE,0xFE, 0xFE,0xFE,0xFF, 0xFE,0xFE,0xFF, 0xFE,0xFE,0xFF, 0xFF,0xFE,0xFF, 0xFE,0xFF,0xFE, 0xFF,0xFE,0xFF, 0xFE,0xFE,0xFF},
    {0x04,0x06,0x05, 0xFE,0x06,0xFE, 0x06,0x06,0x06, 0x04,0x06,0x05, 0xFE,0xFE,0x06, 0x04,0x06,0x05, 0x04,0x06,0x05, 0xFE,0x06,0xFE, 0x04,0x06,0x05, 0x04,0x06,0x05}
};

byte row,nb=0,bc=0;        // general0xFE,0x01,0x07
byte bb[8];                // byte buffer for reading from PROGMEM


// Pin used to receive the data from 433 MHz module
int buttonIn = 6;

int led    = 14;   //Pin A0 - Red LED
int buzzer = 15;   //Pin A1 - Buzzer

char lv_deuce;
char lv_advantage_1;
char lv_advantage_2;
int  lv_score_1;
int  lv_score_2;
int  lv_set_1;
int  lv_set_2;


unsigned long lv_TimerFinal;
unsigned long lv_TimerInitial = 0;
int lv_value = 0;
char lv_loop;

//RCSwitch
RCSwitch mySwitch = RCSwitch();


void setup() {
  //Initialization for input
  pinMode(buttonIn, INPUT);
  
  //Start the pin as output to the LED
  pinMode(led, OUTPUT);

  //Start the pin as output to the buzzer
  pinMode(buzzer,OUTPUT);

  mySwitch.enableReceive(0);  // Interruption 0 is the pin number 2 in Arduino

  lcd.begin(20, 4);  //LCD size 20x4

  for (nb=0; nb<8; nb++ ) {                     // create 8 custom characters
    for (bc=0; bc<8; bc++) bb[bc]= pgm_read_byte( &custom[nb][bc] );
    lcd.createChar ( nb+1, bb );
  }
  
  lcd.clear();  //Initiate the LCD

  //Initiate score with 00 x 00
  printNum(0, 0);  //number 0
  printNum(0, 4);  //number 0
  printNum(0,13);  //number 0
  printNum(0,17);  //number 0

  //Initiate the text for SET
  lcd.setCursor(7, 3);      lcd.print(F("SET"));       lcd.print(F("0"));        lcd.print(F("x"));      lcd.print(F("0"));

  Serial.begin(9600);
}


void loop() {

  //Block used to identify how long the button on RC was pressed
  while (lv_loop != 'X') {

    if (lv_value == 0) {
       lv_TimerFinal = 350000;   }   //Value that you can change in order to adjust the time that separates a short click and a long click
    else {
       lv_TimerInitial = lv_TimerInitial + 1;  
    }

    if (mySwitch.available()) {
      lv_value = lv_value + 1;   //Sum in this variable how long the button on RC is pressed
      //Serial.println(lv_value);
     }

     if (lv_TimerInitial >= lv_TimerFinal ) {
         lv_loop = 'X';
         if (lv_value >= 10) {
           //Switch on the LED that indicates the button on RC was pressed for a long period
           digitalWrite(led, HIGH);
           }
         delay(3000);
         digitalWrite(led, LOW); //Switch off long LED indicator
     }
     mySwitch.resetAvailable();
  }


  statusBotao();   //Call the routine

  //Clear variables
  lv_value = 0;
  lv_loop = ' ';
  lv_TimerInitial = 0;

}


                       ////////////// Subroutines ////////////// 

void statusBotao() {

                    //////////////// Long Click //////////////// 
    if (lv_value >= 10) {
      //Serial.println("Long");   //Button on transmitter was pressed for a long period
                    
      if (lv_deuce != 'X') {
        lv_score_1 = lv_score_1 + 15;

        if (lv_score_1 == 45) {   //Exception to "jump" the count from 30 to 40
        lv_score_1 = 40;
        }

        //Play beep on buzzer that will indicate a new point
        tone(buzzer,440,500);  //Plays 440 Hz for a half second

        //Update score_1 on LCD
        if (lv_score_1 != 55) {
          if (lv_score_1 == 15) {
            printNum(1, 0);  //number 1
            printNum(5, 4);  //number 5
          }
          if (lv_score_1 == 30) {
            printNum(3, 0);  //number 3
            printNum(0, 4);  //number 0
          }
          if (lv_score_1 == 40) {
            printNum(4, 0);  //number 4
            printNum(0, 4);  //number 0
          }
          //Serial.print ("score 1: ");     Serial.println (lv_score_1);
          //Serial.print ("score 2: ");     Serial.println (lv_score_2);
        }  
      }

      if (lv_deuce == 'X' and lv_advantage_1 != 'X')  {
        lv_advantage_1 = 'X';
        lv_advantage_2 = ' ';

        if (lv_score_2 == 1) {
          lv_score_2 = 0;
          lv_advantage_1 = ' ';
          lcd.clear();
          printNum(0, 2);  //number 0
          printNum(0,15);  //number 0
          
        }
        else {
          //Print 0 (advantage) for one player and 1 to the other.
          lv_score_1 = 1;
          lv_score_2 = 0;
          lcd.clear();
          printNum(1, 2);  //number 1
          printNum(0,15);  //number 0
          
        }

        //Play beep on buzzer that will indicate a new point
        tone(buzzer,440,500);  //Plays 440 Hz for a half second

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);
        //Serial.print ("score 1: ");     Serial.println (lv_score_1);
        //Serial.print ("score 2: ");     Serial.println (lv_score_2);
      }
      else if (lv_deuce == 'X' and lv_advantage_1 == 'X')  {

        lv_set_1 = lv_set_1 + 1;

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);
        
        //Serial.print ("SET jogador 1: ");   Serial.println (lv_set_1);  //Imprimir no LCD
        //Serial.println ("Player 1 is the champion");

        //Plays a different sound to indicate that the game is over
        tone(buzzer,750,250);  //Plays 750 Hz for a half second
        delay(500);
        tone(buzzer,750,250);  //Plays 750 Hz for a half second

        //Wait up to 2 seconds.
        delay(2000);

        //Initiate score.
        lv_score_1 = 0;
        lv_score_2 = 0;
        lv_deuce = ' ';

        lcd.clear();

        //Initiate the score with 00 x 00
        printNum(0, 0);  //number 0
        printNum(0, 4);  //number 0
        printNum(0,13);  //number 0    
        printNum(0,17);  //number 0

        //Initiate the text for SET
        lcd.setCursor(7, 3);      lcd.print(F("SET"));       lcd.print(lv_set_1);        lcd.print(F("x"));      lcd.print(lv_set_2);
        //Serial.print ("score 1: ");      Serial.println (lv_score_1);
        //Serial.print ("score 2: ");      Serial.println (lv_score_2);

      }


      //DEUCE 40x40
      if ((lv_score_1 == 40 and lv_score_2 == 40) and lv_deuce != 'X') {   //Deuce
        lv_deuce = 'X';
        //Serial.println ("DEUCE");  //Print on LCD
        lcd.clear();        
        printNum(0, 2);  //number 0
        printNum(0,15);  //number 0

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);
      }
                   

      if ((lv_score_1 == 55 and lv_score_2 < 40) and lv_deuce != 'X')    {
        //Update a variable to the SET that will be shown in a separated space on LCD
        lv_set_1 = lv_set_1 + 1;

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);
        
        //Serial.print ("SET jogador 1: ");   Serial.println (lv_set_1);   //Imprimir no LCD
        //Serial.println ("Player 1 is the champion");

        //Plays a different sound to indicate that the game is over
        tone(buzzer,750,250);  //Plays 750 Hz for a half second
        delay(500);
        tone(buzzer,750,250);  //Plays 750 Hz for a half second
       
        //wait up to 2 seconds
        delay(2000);
       
        //Initiate score_1 and score_2.
        lv_score_1 = 0;
        lv_score_2 = 0;

        lcd.clear();

        //Inicia score com 00 x 00
        printNum(0, 0);  //number 0
        printNum(0, 4);  //number 0
        printNum(0,13);  //number 0    
        printNum(0,17);  //number 0

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);
        //Serial.print ("score 1: ");       Serial.println (lv_score_1);
        //Serial.print ("score 2: ");       Serial.println (lv_score_2);  
        lv_deuce = ' ';
        lv_advantage_1 = ' ';
        lv_advantage_2 = ' ';
      }

    }


    else {

                  //////////////// Short Click ////////////////          
      //Serial.println("short");   //Button on transmitter was pressed for a short period

      if (lv_deuce != 'X') {
        lv_score_2 = lv_score_2 + 15;

        if (lv_score_2 == 45)  {   //Exception to "jump" the count from 30 to 40
          lv_score_2 = 40;
        }

        //Play beep on buzzer that will indicate a new point
        tone(buzzer,1000,500);  //Plays 1000 Hz for a half second
        
        //Atualiza no display o score_2
        if (lv_score_2 != 55) {

          if (lv_score_2 == 15) {
            printNum(1,13);  //number 1
            printNum(5,17);  //number 5
          }
          if (lv_score_2 == 30) {
            printNum(3,13);  //number 3
            printNum(0,17);  //number 0
          }          
          if (lv_score_2 == 40) {
            printNum(4,13);  //number 4
            printNum(0,17);  //number 0
          }                    
          //Serial.print ("score 1: ");     Serial.println (lv_score_1);
          //Serial.print ("score 2: ");     Serial.println (lv_score_2);
        }
      }


      if (lv_deuce == 'X' and lv_advantage_2 != 'X')  {
        lv_advantage_2 = 'X';
        lv_advantage_1 = ' ';

        if (lv_score_1 == 1) {
            lv_score_1 = 0;
            lv_advantage_2 = ' ';
            lcd.clear();
            printNum(0, 2);  //number 0
            printNum(0,15);  //number 1
            
        }
        else {
          //Prints 0 (advantage) to one player and 1 to the other player
          lv_score_1 = 0;
          lv_score_2 = 1;
          lcd.clear();
          printNum(0, 2);  //number 0
          printNum(1,15);  //number 1

        }

        //Play beep on buzzer that will indicate a new point
        tone(buzzer,1000,500);  //Plays 1000 Hz for a half second

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);        
        //Serial.print ("score 1: ");     Serial.println (lv_score_1);
        //Serial.print ("score 2: ");     Serial.println (lv_score_2);
      }

      else if (lv_deuce == 'X' and lv_advantage_2 == 'X')  {

        lv_set_2 = lv_set_2 + 1;

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);    
            
        //Serial.print ("SET jogador 2: ");   Serial.println (lv_set_2);   //Print on LCD
        //Serial.println ("Player 2 is the champion");

        //Plays a different sound to indicate that the game is over
        tone(buzzer,750,250);  //Plays 750 Hz for a half second
        delay(500);
        tone(buzzer,750,250);  //Plays 750 Hz for a half second
                
        //Wait up to 2 seconds.
        delay(2000);    
    
        //Initiate score.
        lv_score_1 = 0;
        lv_score_2 = 0;
        lv_deuce = ' ';

        lcd.clear();

        //Initiate the score with 00 x 00
        printNum(0, 0);  //number 0
        printNum(0, 4);  //number 0
        printNum(0,13);  //number 0    
        printNum(0,17);  //number 0

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);      lcd.print(F("SET"));       lcd.print(lv_set_1);        lcd.print(F("x"));      lcd.print(lv_set_2);
        //Serial.print ("score 1: ");    Serial.println (lv_score_1);
        //Serial.print ("score 2: ");    Serial.println (lv_score_2);
      }


      //DEUCE 40x40
      if ((lv_score_2 == 40 and lv_score_1 == 40) and (lv_deuce != 'X'))  {
        lv_deuce = 'X';
        //Serial.println ("DEUCE");   //Print on LCD
        lcd.clear();
        printNum(0, 2);  //number 0
        printNum(0,15);  //number 0            

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);         lcd.print(F("SET"));         lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);        
      }

            
     
      if ((lv_score_2 == 55 and lv_score_1 < 40) and lv_deuce != 'X')  {
        //Update a variable to the SET that will be shown in a separated space on LCD
        lv_set_2 = lv_set_2 + 1;

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);
        //Serial.print ("SET jogador 2: ");   Serial.println (lv_set_2);   //Imprimir no LCD
        //Serial.println ("Player 2 is the champion");

        //Plays a different sound to indicate that the game is over
        tone(buzzer,750,250);  //Plays 750 Hz for a half second
        delay(500);
        tone(buzzer,750,250);  //Plays 750 Hz for a half second
              
        //wait up to 2 seconds
        delay(2000);
      
        //Initiate score_1 and score_2.
        lv_score_1 = 0;
        lv_score_2 = 0;

        lcd.clear();

        //Initiate score with 00 x 00
        printNum(0, 0);  //number 0
        printNum(0, 4);  //number 0
        printNum(0,13);  //number 0    
        printNum(0,17);  //number 0

        //Print on LCD the texto to the SET
        lcd.setCursor(7, 3);        lcd.print(F("SET"));        lcd.print(lv_set_1);        lcd.print(F("x"));        lcd.print(lv_set_2);
        //Serial.print ("score 1: ");      Serial.println (lv_score_1);
        //Serial.print ("score 2: ");      Serial.println (lv_score_2);      
        lv_deuce = ' ';
        lv_advantage_1 = ' ';
        lv_advantage_2 = ' ';          
      }

    }

  }

  void printNum(byte digit, byte leftAdjust) {
     for(row=0; row<4; row++) {
       lcd.setCursor(leftAdjust,row);               
       for(byte num=digit*3; num <digit*3+3; num++) {
         lcd.write(pgm_read_byte( &bn4[row][num]) );
       }
     }
  }
  
